using System;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Collections.Generic;
using System.Text;

namespace MutexSample
{
    class Program
    {
        static int sharedNumber = 42;
        static Mutex localMut = new Mutex();
        static bool isNew;
        static Mutex globalMut = new Mutex(true, "Mutex Demo Global Mutex", out isNew);        
        static void Main(string[] args)
        {
            if (!isNew)
            {
                Console.WriteLine("This application is already running, shutting additional instance down.");
                return;
            }
            // spin off a bunch of threads to perform 
            // processing on a shared resource
            Thread[] workers = new Thread[20];
            for (int i = 0; i < 20; i++)
            {
                Thread worker = new Thread(new ThreadStart(DoWork));
                workers[i] = worker;
                worker.Start();
            }

            foreach (Thread workerThread in workers)
                workerThread.Join();
            Console.WriteLine("All work finished, new value of shared resource is {0}", sharedNumber);
            Console.ReadLine();
            globalMut.ReleaseMutex();
        }

        static void DoWork()
        {
            // sit and wait until its OK to access
            // the shared resource
            localMut.WaitOne();

            // modify shared resource
            // multiple lines of code to modify
            // to show consistent state of data
            // within Mutex-protected block
            Console.WriteLine("Accessing protected resource...");
            sharedNumber += 2;
            sharedNumber -= 1;

            localMut.ReleaseMutex();
        }
    }
}
